//
//  MyOpenGLView.m
//  LorenzCocoa
//
//  Created by 川上 博 on 2015/11/16.
//  Copyright © 2015年 Hiroshi Kawakami. All rights reserved.
//

#import "MyOpenGLView.h"
#import <GLUT/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define DATA_SIZE 10000
#define CARS	2
#define step	0.005

NSTimer *animTimer;
static double orbit[DATA_SIZE][3], cars[CARS][3];
double x[3]={2.0, 2.0, 2.0};

void func(double x[], double f[], double t)
{
    f[0] =10.0*(x[1]-x[0]);
    f[1] =28.0*x[0]-x[1]-x[0]*x[2];
    f[2] =-2.67*x[2]+x[0]*x[1];
}

void runge(int n, double h, double x[], double t)
{
    double k1[n], k2[n], k3[n], k4[n], xtemp[n];
    int i;

    func(x,k1,t);
    for (i=0; i<n; i++) xtemp[i]=x[i]+k1[i]/2.0*h;
    t+=h/2.0;
    func(xtemp,k2,t);
    for (i=0; i<n; i++) xtemp[i]=x[i]+k2[i]/2.0*h;
    func(xtemp,k3,t);
    for (i=0; i<n; i++) xtemp[i]=x[i]+k3[i]*h;
    t+=h/2.0;
    func(xtemp,k4,t);
    for (i=0; i<n; i++) x[i]+=(k1[i]+2.0*(k2[i]+k3[i])+k4[i])/6.0*h;
}

void makeOrbit(void)
{
    int  i, j;

    for(i=0; i<300; i++) runge(3, step, x, 0.0);
    for(i=0; i < DATA_SIZE; i++){
        runge(3, step, x, 0.0);
        for (j=0; j<3; j++) orbit[i][j] = x[j];
    }
}

void moveCars(void)
{
    int i;

    for(i=0; i<CARS; i++) runge(3, step, cars[i], 0.0);
}

void initCars(void)
{
    int i;

    for(i=0; i<CARS; i++){
        cars[i][0] = 10.0*((double) rand()/(RAND_MAX) - 0.5);
        cars[i][1] = 0.0;
        cars[i][2] = 30.0;
    }
    moveCars();
}

void myDraw(void)
{
    int i;

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0.0,70.0,0.0, 0.0,0.0,25.0, 0.0,0.0,1.0);

    glPushMatrix();
    glBegin(GL_LINE_STRIP);
    glColor3f(0.0, 1.0, 1.0);
    for(i=0; i < DATA_SIZE; i++) glVertex3dv (orbit[i]);
    glEnd();
    glPopMatrix();

    for(i=0; i<CARS; i++){
        glPushMatrix();
        glTranslated(cars[i][0],cars[i][1],cars[i][2]);
        glRotated(90.0,1.0,0.0,0.0);
        if(i==0) glColor3f(1.0,0.0,0.0);
        else glColor3f(1.0,1.0,0.0);
        glutSolidSphere(0.5,10,10);
        glPopMatrix();
    }
}

@implementation MyOpenGLView
- (void)prepareOpenGL
{
    glClearColor(0.0, 0.0, 0.0, 0.0); //background color
    glEnable(GL_DEPTH_TEST);
    makeOrbit();
    initCars();

    animTimer=[NSTimer scheduledTimerWithTimeInterval:0.005
                                                target:self
                                              selector:@selector(drawAnimation:)
                                              userInfo:nil
                                               repeats:YES];
}

- (void)drawAnimation:(NSTimer *)aTimer
{
    moveCars();
    //	glutPostRedisplay();
    [self setNeedsDisplay:YES];
}


- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (void)keyDown:(NSEvent *)theEvent
{
    switch([[theEvent characters] characterAtIndex:0]){
        case 'q':case 'Q':case '\033':
            exit(0);
            break;
        default:
            break;
    }
}

- (void)reshape
{
    NSRect rect;
    
    [[self openGLContext] update];
    rect=[self bounds];
    glViewport(0,0,rect.size.width, rect.size.height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0, (double)rect.size.width/(double)rect.size.height, 2.0, 120.0);
}

- (void)drawRect:(NSRect)rect 
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    myDraw();
    glFlush();
}
@end
